// Updatecrossrefs.js
// Updates cross references made with hyperlinks
// the hyperlink destination is where the reference refers to
// the hyperlink sourcetext is the text that will be updated (like see xx)
// If a book is open, the book will be handled,
// if no book is open, the active document will be handled.
// Made by Teus de Jong, last version April 10, 2006
// Revisions by Dave Saunders January 22, 2006 and March 20, 2006
// Thanks Dave!
if (app.books.length == 0){
	if (app.documents.length == 0){
		errorExit('No documents are open');
	} else {
		nrofdocs = 1;
	}
} else {
	nrofdocs = app.books[0].bookContents.length;
	for (i = nrofdocs - 1; i > -1; i--) {
		app.open(File(app.books[0].bookContents[i].fullName));
	}
}
doc = app.activeDocument;
for (d = 1; d <= nrofdocs; d++){
	for (i = 0; i < doc.hyperlinks.length; i++){
		hyper = doc.hyperlinks[i];
  		// leave ID's own internal markers alone 
		if (hyper.name.substring(0, 1) == '.'){continue} 
  		// leave hyperlinks to URLs alone
  		try {
			if (hyper.destination.constructor.name == 'HyperlinkURLDestination'){continue}
		} catch (e) {
			hyper.showSource();
			errorExit('Selected hyperlink has undefined destination');
		}
		if (hyper.destination.constructor.name == 'HyperlinkTextDestination'){
			s = getParentPageName(hyper.destination.destinationText);
 		} else { 
 			// try/catch added to allow for hyperlinks that have lost their destination -- Dave
 			try {
		  		s = hyper.destination.destinationPage.name;
		  	} catch (e) { continue }
		} 
		if (s != ''){
			// Check to see if update needed; saves a lot of time when processing large books -- Dave
			if (hyper.source.sourceText.contents != s) {
				hyper.source.sourceText.contents = s;
			}
		}
	}
	if (d >= nrofdocs){
		break;
	}
	doc = app.open(app.books[0].bookContents[d].fullName);
}  

function getParentPageName(txt){
	try {  
		var myFrame = getParentTextFrame(txt); 
		var pag = myFrame.parent; 
		while (pag.constructor.name != 'Page'){ 
			// in case the reference is in an inline, the parent of the frame is a character
			if (pag.constructor.name == 'Character'){
				pag = getParentTextFrame(pag);
			}
			// in case the chain goes sour, back out
			if (pag.constructor.name == 'Application'){
				throw 'Error';
			}
			pag = pag.parent;
		}
		s = pag.name; 
	} 
	catch (e) { 
		alert('Could not catch page.\nCheck destination names and existence.'); 
		s = ''; 
	}
	return(s);
}

function getParentTextFrame(x){
	if(app.version == 3){ 
		return(x.parentTextFrame);
	}
	else { 
		return(x.parentTextFrames[0]);
	}
}

function errorExit(s){
	alert(s);
	exit();
}